%  Jacobi method to approximate x of Ax=y.
%
%  Author: Martin Krosche
%          Institute of Scientific Computing
%          Technische Universitaet Braunschweig
%          Braunschweig, Germany
%
%  Copyright (c) 2008 Martin Krosche. All rights reserved. No warranty. No
%  liability.
%
%  [X, ERR] = JACOBI( A, Y, X0, ITER ) returns X by performing ITER
%  iterations and taking starting vector X0. ERR is the L2-norm of the residuum.

function [x, err] = jacobi( A, y, x0, iter )

    %  get diagonal in sparse format.
    n    = length( A );
    D    = spdiags( diag( A ), 0, n, n );

    %  get lower and upper triangular.
    L = tril( A, -1 );
    U = triu( A, 1 );
    LU = L+U;

    invD = inv( D );
    M = -invD * LU;
    g = invD * y;

    %  iterate...
    x = x0;
    for i=1:iter
        x = M*x + g;
    end;

    err = norm( ( A*x-y ), 2 );
